{ config, lib, pkgs, ... }:
{
  config = lib.mkIf config.home.desktop.enable {
    programs.firefox = {
      enable = true;
      package = pkgs.librewolf;
      profiles.default = {
        isDefault = true;

        containersForce = true;
        containers = {
          home = {
            color = "purple";
            icon = "fruit";
            id = 1;
          };
          school = {
            color = "red";
            icon = "briefcase";
            id = 2;
          };
        };

        bookmarks = {
          force = true;
          settings = [
            {
              name = "NixFox";
              url = "https://www.nixfox.ca";
            }
            {
              name = "Luna";
              url = "https://www.lunamoonlight.xyz";
            }
            {
              name = "Freecorn";
              url = "https://freecorn1854.win";
            }
            {
              name = "Bloxelcom";
              url = "https://www.bloxelcom.net";
            }
            {
              name = "YouTube";
              url = "https://www.kernel.org";
            }
            {
              name = "NixOS Wiki";
              url = "https://nixos.wiki";
            }
            {
              name = "Kernel";
              url = "https://www.kernel.org";
            }
          ];
        };

        extensions = {
          force = true;
          packages = with pkgs.nur.repos.rycee.firefox-addons; [
            bitwarden
            darkreader
            return-youtube-dislikes
            simple-tab-groups
            sponsorblock
            ublock-origin
            violentmonkey
          ];
        };

        search = {
          force = true;
          default = "ecosia";
          engines = {
            "ecosia" = {
              urls = [{
                template = "https://www.ecosia.org/search";
                params = [{
                  name = "q";
                  value = "{searchTerms}";
                }];
              }];
              definedAliases = [ "@e" ];
            };

            "nixpkgs" = {
              urls = [{
                template = "https://search.nixos.org/packages";
                params = [
                  {
                    name = "type";
                    value = "packages";
                  }
                  {
                    name = "query";
                    value = "{searchTerms}";
                  }
                ];
              }];
              definedAliases = [ "@p" ];
            };

            "mynixos" = {
              urls = [{
                template = "https://mynixos.com/search";
                params = [{
                  name = "q";
                  value = "{searchTerms}";
                }];
              }];
              definedAliases = [ "@my" ];
            };

            "youtube" = {
              urls = [{
                template = "https://www.youtube.com/results";
                params = [{
                  name = "search_query";
                  value = "{searchTerms}";
                }];
              }];
              definedAliases = [ "@y" ];
            };

            "spotify" = {
              urls = [{ template = "https://open.spotify.com/search/{searchTerms}"; }];
              definedAliases = [ "@s" ];
            };
          };
        };

        settings = {
          force = true;

          # Generic
          "browser.download.useDownloadDir" = true;
          "browser.shell.checkDefaultBrowser" = false;
          "browser.startup.page" = 3;
          "browser.urlbar.speculativeConnect.enabled" = false;

          # Hardware tweaks
          "clipboard.autocopy" = false;
          "general.autoScroll" = true;
          "middlemouse.paste" = false;
          "toolkit.tabbox.switchByScrolling" = true;
          "webgl.disabled" = false;

          # Fonts
          "font.name.monospace.x-western" = config.vars.fonts.mono;
          "font.name.sans-serif.x-western" = config.vars.fonts.main;
          "font.name.serif.x-western" = config.vars.fonts.main;

          # Theming and visibility
          "browser.aboutConfig.showWarning" = false;
          "browser.compactmode.show" = true;
          "browser.tabs.inTitlebar" = 1;
          "browser.theme.content-theme" = 0;
          "browser.theme.toolbar-theme" = 0;
          "browser.toolbars.bookmarks.visibility" = "never";
          "browser.uidensity" = 1;
          "extensions.activeThemeID" = "firefox-compact-dark@mozilla.org";
          "toolkit.legacyUserProfileCustomizations.stylesheets" = true;

          # New tab
          "browser.newtabpage.activity-stream.asrouter.userprefs.cfr.addons" = false;
          "browser.newtabpage.activity-stream.asrouter.userprefs.cfr.features" = false;
          "browser.newtabpage.activity-stream.showSponsoredTopSites" = false;
          "browser.newtabpage.enabled" = false;

          # Safe browsing
          "browser.safebrowsing.blockedURIs.enabled" = false;
          "browser.safebrowsing.downloads.enabled" = false;
          "browser.safebrowsing.downloads.remote.block_potentially_unwanted" = false;
          "browser.safebrowsing.downloads.remote.block_uncommon" = false;
          "browser.safebrowsing.downloads.remote.enabled" = false;
          "browser.safebrowsing.malware.enabled" = false;
          "browser.safebrowsing.phishing.enabled" = false;
          "browser.safebrowsing.provider.google.gethashURL" = false;
          "browser.safebrowsing.provider.google.updateURL" = false;
          "browser.safebrowsing.provider.google4.gethashURL" = false;
          "browser.safebrowsing.provider.google4.updateURL" = false;

          # Extensions
          "extensions.formautofill.addresses.enabled" = false;
          "extensions.formautofill.creditCards.enabled" = false;
          "extensions.getAddons.showPane" = false;
          "extensions.htmlaboutaddons.discover.enabled" = false;
          "extensions.htmlaboutaddons.recommendations.enabled" = false;
          "extensions.pocket.enabled" = false;

          # Force extensions to work
          "extensions.autoDisableScopes" = 0;
          "extensions.enabledScopes" = 15;

          # Privacy
          "privacy.clearOnShutdown.cookies" = false;
          "privacy.clearOnShutdown_v2.cookiesAndStorage" = false;
          "privacy.fingerprintingProtection" = true;
          "privacy.globalprivacycontrol.enabled" = true;
          "privacy.globalprivacycontrol.functionality.enabled" = true;
          "privacy.resistFingerprinting" = true;

          # Tracking URL blocking
          "urlclassifier.features.socialtracking.skipURLs" = "*.instagram.com, *.twitter.com, *.twimg.com";
          "urlclassifier.trackingSkipURLs" = "*.reddit.com, *.twitter.com, *.twimg.com, *.tiktok.com";

          # Network tracking
          "browser.contentblocking.category" = "strict";
          "network.cookie.cookieBehavior" = 1;
          "network.http.referer.XOriginPolicy" = 2;
          "network.http.referer.XOriginTrimmingPolicy" = 2;
          "network.http.referer.trimmingPolicy" = 2;
          "network.trr.mode" = 3;
          "network.trr.uri" = "https://wikimedia-dns.org/dns-query";

          # HTTPS upgrading
          "dom.security.https_only_mode" = true;
          "dom.security.https_only_mode_ever_enabled" = true;
          "dom.security.https_only_mode_ever_enabled_pbm" = true;

          # Sensors
          "device.sensors.motion.enabled" = false;
          "dom.battery.enabled" = false;
          "dom.private-attribution.submission.enabled" = false;

          # Telemetry
          "browser.discovery.enabled" = false;
          "browser.send_pings" = false;
          "datareporting.healthreport.uploadEnabled" = false;
          "identity.fxaccounts.enabled" = false;
          "security.OCSP.require" = true;
          "services.sync.engine.addresses.available" = false;
          "svg.context-properties.content.enabled" = true;
        };

        userChrome = ''
          :root {
            --tab-active-bg-color: #${config.vars.colors.prime};
            --tab-hover-bg-color: #${config.vars.colors.accent};
            --tab-inactive-bg-color: #${config.vars.colors.dark};
            --tab-active-fg-fallback-color: #FFFFFF;
            --tab-inactive-fg-fallback-color: #${config.vars.colors.text};
            --urlbar-focused-bg-color: #${config.vars.colors.dark};
            --urlbar-not-focused-bg-color: #${config.vars.colors.dark};
            --toolbar-bgcolor: #${config.vars.colors.dark} !important;
            --tab-font: '${config.vars.fonts.main}';
            --urlbar-font: '${config.vars.fonts.main}';
            ${builtins.readFile ./quteFox.css}
        '';
      };
    };

    # Symlinks to Librewolf
    home = {
      file.".librewolf".source = config.lib.file.mkOutOfStoreSymlink "${config.home.homeDirectory}/.mozilla/firefox";
      persistence."${config.xdg.userDirs.extraConfig.XDG_PERSIST_DIR}".directories = [ "browser/.mozilla/firefox/default" ];
    };
  };
}
